from random import randint;import pygame,random;from pygame_gui.elements.ui_window import UIWindow;from pygame_gui.elements.ui_image import UIImage;import os;from math import cos, sin, tan, pi, floor, ceil, sqrt;two_pi = pi * 2;half_pi = pi * 0.5;three_half_pi = pi * 1.5;NUMBER_OF_RAYS = 320
class DisjointSet:
	class Element:
		def __init__(self, key):self.key = key;self.parent = self;self.rank = 0
		def __eq__(self, other):return self.key ==  other.key
		def __ne__(self, other):return self.key != other.key
	def __init__(self):self.tree = {}
	def make_set(self, key):
		e = self.Element(key)
		if not key in self.tree.keys():self.tree[key] = e
	def find(self, key):
		if key in self.tree.keys():element = self.tree[key]
		if key in self.tree.keys() and element.parent != element:element.parent  = self.find(element.parent.key)
		if key in self.tree.keys():return element.parent
	def union(self, element_a, element_b):
		root_a = self.find(element_a.key);root_b = self.find(element_b.key)
		if root_a != root_b and root_a.rank < root_b.rank:root_a.parent = root_b
		elif root_a != root_b and root_a.rank > root_b.rank:root_b.parent = root_a
		elif root_a != root_b:root_b.parent = root_a;root_a.rank+=1
class Maze:
	def __init__(self, width):
		self.width = width;self.height = width;self.seed = random.random()*10000;self.grid = [[(width*row + col) for row in range(0,width)] for col in range(0, width)];self.portals = {};self.kruskalize()
	def __str__(self):
		s=''
		for i in range(self.width):s+=str(random.randint(1,2))+str(random.randint(1,2))
		s+=str(random.randint(1,2))+'\n'
		for row in range(0,self.height):
			s+=str(random.randint(1,2))+'0'
			for col in range(1, self.width):s+='0'*(self.grid[col-1][row] in self.portals[self.grid[col][row]])+str(random.randint(1,2))*(self.grid[col-1][row] not in self.portals[self.grid[col][row]])+'0'
			s+=str(random.randint(1,2))+'\n'
			for col in range(0, self.width):key = self.grid[col][row];s+=str(random.randint(1,2))+'0'*(row+1 < self.height and self.grid[col][row+1] in self.portals[key])+str(random.randint(1,2))*(row+1 >= self.height or self.grid[col][row+1] not in self.portals[key])
			s+=str(random.randint(1,2)) +'\n'
		return s
	def kruskalize(self):
		edges_ordered = [ ]
		for row in range(0, self.height):
			for col in range(0, self.width):
				cell = (col, row);left_cell = (col-1, row);down_cell = (col, row-1);near = []
				if col > 0:near.append((left_cell, cell))
				if row > 0:near.append( (down_cell, cell))
				edges_ordered.extend(near)
		random.seed(self.seed);edges = []
		while len(edges_ordered) > 0:edges.append(edges_ordered.pop(random.randint(0,len(edges_ordered))-1))
		disjoint_set = DisjointSet()
		for row in range(0, self.height):
			for col  in range(0,self.width):key = self.grid[col][row];disjoint_set.make_set(key);self.portals[key] = {}
		edge_count = 0;key_count = self.grid[self.width-1][self.height-1]
		while edge_count < key_count:
			edge = edges.pop();key_a = self.grid[edge[0][0]][edge[0][1]];key_b = self.grid[edge[1][0]][edge[1][1]];set_a = disjoint_set.find(key_a);set_b = disjoint_set.find(key_b)
			if set_a != set_b:edge_count+=1;self.portals[key_a][key_b] = True;self.portals[key_b][key_a] = True;disjoint_set.union(set_a, set_b)
	def list(self):return [list(i) for i in str(self).split('\n')][:-1]
class Maze3D:
    def __init__(self, size):self.map=Maze(42).list();app_path = os.path.dirname(os.path.abspath(__file__));assets_path = app_path + "/assets";self.size = size;self.width = self.size[0];self.height = self.size[1];self.walldark = [None];self.walllight = [None];self.walldark.append(pygame.image.load(assets_path + "/1.png").convert());self.walllight.append(pygame.image.load(assets_path + "/2.png").convert());self.walldark.append(pygame.image.load(assets_path + "/3.png").convert());self.walllight.append(pygame.image.load(assets_path + "/4.png").convert());self.textureSize = self.walldark[1].get_size();self.bg_img = pygame.image.load(assets_path + "/background.png").convert();self.bg_img = pygame.transform.scale(self.bg_img, size);self.playerX = 1.5;self.playerY = 1.5;self.playerDir = 0.0;self.playerStep = 0.05;self.angleStep = 5.0 * pi / 180.0;self.toggleTurnLeft = False;self.toggleTurnRight = False;self.toggleMoveFw = False;self.toggleMoveBw = False;self.defaultHeight = 400.0;self.numRays = NUMBER_OF_RAYS;self.viewAngle = 65 * pi / 180.0;self.rayAngleStep = self.viewAngle / (self.numRays - 1);self.columnWidth = self.width / (self.numRays - 1)
    def updatePos(self):
        if self.toggleTurnLeft:self.playerDir = (self.playerDir + self.angleStep) % (two_pi)
        if self.toggleTurnRight:self.playerDir = (self.playerDir - self.angleStep) % (two_pi)
        if self.toggleMoveFw:self.move(1)
        if self.toggleMoveBw:self.move(-1)
    def move(self, dir):
        flag = False;x, y = self.playerX, self.playerY;nx = x + self.playerStep * dir * cos(self.playerDir);ny = y - self.playerStep * dir * sin(self.playerDir)
        if self.map[floor(ny)][floor(nx)] == 0:self.playerX, self.playerY = nx, ny;return
        if floor(x) == floor(nx) and floor(y) == floor(ny) + 1:self.playerX, self.playerY = nx, ceil(ny) + 0.01;return
        if floor(x) == floor(nx) and floor(y) == floor(ny) - 1:self.playerX, self.playerY = nx, floor(ny) - 0.01;return
        if floor(x) == floor(nx) + 1 and floor(y) == floor(ny):self.playerX, self.playerY = ceil(nx) + 0.01, ny;return
        if floor(x) == floor(nx) - 1 and floor(y) == floor(ny):self.playerX, self.playerY = floor(nx) - 0.01, ny;return
        if self.map[floor(ny)][floor(x)] == 0 and self.map[floor(y)][floor(nx)] == 0:flag = True
        self.playerX, self.playerY = nx, ny
        if floor(x) == floor(nx) + 1 and floor(y) == floor(ny) + 1:
            if self.map[floor(ny)][floor(x)] != 0 or flag:self.playerY = ceil(ny) + 0.01
            if self.map[floor(y)][floor(nx)] != 0 or flag:self.playerX = ceil(nx) + 0.01
            return
        if floor(x) == floor(nx) - 1 and floor(y) == floor(ny) + 1:
            if self.map[floor(ny)][floor(x)] != 0 or flag:self.playerY = ceil(ny) + 0.01
            if self.map[floor(y)][floor(nx)] != 0 or flag:self.playerX = floor(nx) - 0.01
            return
        if floor(x) == floor(nx) - 1 and floor(y) == floor(ny) - 1:
            if self.map[floor(ny)][floor(x)] != 0 or flag:self.playerY = floor(ny) - 0.01
            if self.map[floor(y)][floor(nx)] != 0 or flag:self.playerX = floor(nx) - 0.01
            return
        if floor(x) == floor(nx) + 1 and floor(y) == floor(ny) - 1:
            if self.map[floor(ny)][floor(x)] != 0 or flag:self.playerY = floor(ny) - 0.01
            if self.map[floor(y)][floor(nx)] != 0 or flag:self.playerX = ceil(nx) + 0.01
            return
    def process_event(self, event):
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_LEFT:self.toggleTurnLeft = True;return True
            elif event.key == pygame.K_RIGHT:self.toggleTurnRight = True;return True
            elif event.key == pygame.K_UP:self.toggleMoveFw = True;return True
            elif event.key == pygame.K_DOWN:self.toggleMoveBw = True;return True
            return False
        if event.type != pygame.KEYUP:return False
        if event.key == pygame.K_LEFT:self.toggleTurnLeft = False;return True
        elif event.key == pygame.K_RIGHT:self.toggleTurnRight = False;return True
        elif event.key == pygame.K_UP:self.toggleMoveFw = False;return True
        elif event.key == pygame.K_DOWN:self.toggleMoveBw = False;return True
        return False
    def draw(self, surface):
        surface.blit(self.bg_img, (0, 0))
        for r in range(self.numRays):
            rayAngle = (self.playerDir + (self.viewAngle / 2) - r * self.rayAngleStep) % (two_pi);distH = 100000;propH = 0
            if rayAngle != half_pi and rayAngle != three_half_pi:
                if rayAngle < half_pi or rayAngle > three_half_pi:tanRayAngle = tan(rayAngle);x = ceil(self.playerX);y = self.playerY - tanRayAngle * (x - self.playerX);dir = 1
                else:tanRayAngle = tan(rayAngle);x = floor(self.playerX);y = self.playerY - tanRayAngle * (x - self.playerX);dir = -1
                while y > 0 and y < len(self.map) and self.map[floor(y)][floor(x + 0.1 * dir)] == 0:x += dir;y -= dir * tanRayAngle
                if y > 0 and y < len(self.map):wallTypeH = self.map[floor(y)][floor(x + 0.1 * dir)];distH = sqrt((self.playerX - x) * (self.playerX - x) + (self.playerY - y) * (self.playerY - y));propH = y % 1 if dir == 1 else 1 - (y % 1)
            distV = 100000;propV = 0
            if rayAngle != 0 and rayAngle != pi:
                if rayAngle < pi:arctanRayAngle = 1.0 / tan(rayAngle);y = floor(self.playerY);x = self.playerX + arctanRayAngle * (self.playerY - y);dir = -1
                else:arctanRayAngle = 1.0 / tan(rayAngle);y = ceil(self.playerY);x = self.playerX + arctanRayAngle * (self.playerY - y);dir = 1
                while x > 0 and x < len(self.map[0]) and self.map[floor(y + 0.1 * dir)][floor(x)] == 0:y += dir;x -= dir * arctanRayAngle
                if x > 0 and x < len(self.map[0]):wallTypeV = self.map[floor(y + 0.1 * dir)][floor(x)];distV = sqrt((self.playerX - x) * (self.playerX - x) + (self.playerY - y) * (self.playerY - y));propV = x % 1 if dir == -1 else 1 - (x % 1)
            if distH < distV:dist = distH;prop = propH;wallImg = self.walllight[wallTypeH]
            else:coeffLight = 1.0;dist = distV;prop = propV;wallImg = self.walldark[wallTypeV]
            z = max(0.01, dist * cos((rayAngle - self.playerDir)));calculatedHeight = self.defaultHeight * (1 / z)
            if calculatedHeight > self.height:h = self.height;cropHeight = self.height / calculatedHeight * self.textureSize[1];offset = (self.textureSize[1] - cropHeight) / 2
            else:h = calculatedHeight;cropHeight = self.textureSize[1];offset = 0
            cropArea = (self.textureSize[0] * prop, offset, 1, cropHeight);croppedImg = wallImg.subsurface(cropArea);sliceShape = (ceil(self.columnWidth), floor(h + 0.5));wallSliceImg = pygame.transform.scale(croppedImg, sliceShape);surface.blit(wallSliceImg, (r * self.columnWidth, (self.height - h) / 2))
class Maze3DWindow(UIWindow):
    def __init__(self, position, ui_manager):super().__init__(pygame.Rect(position, (640, 480)),ui_manager,window_display_title="Maze 3D",object_id="#maze3d_window",);game_surface_size = self.get_container().get_size();self.game_surface_element = UIImage(pygame.Rect((0, 0), game_surface_size),pygame.Surface(game_surface_size).convert(),manager=ui_manager,container=self,parent_element=self,);self.maze3d = Maze3D(game_surface_size)
    def process_event(self, event):super().process_event(event);return self.maze3d.process_event(event)
    def update(self, time_delta):super().update(time_delta);self.maze3d.updatePos();self.maze3d.draw(self.game_surface_element.image)
def load(manager, params):
    pos = (100, 100)
    if params is not None and len(params) > 0:pos = params[0]
    Maze3DWindow(pos, manager)



import os,sys,time,threading,keyboard,pygame,pygame_gui,importlib;sys.dont_write_bytecode=True;os.environ['PYGAME_HIDE_SUPPORT_PROMPT']='1';from pygame_gui.elements import UIPanel,UIButton;PANEL_LAYER = 10;BUTTON_DIMS = (200, 25);pygame.display.set_icon(pygame.image.load(os.path.join(os.path.abspath(os.path.dirname(__file__)),'image.jpg')))
class Menu(UIPanel):
    manager = None;pos = None;path = None;child = None;elements = None
    def __init__(self, manager, pos, path, elements, loadfunc):super().__init__(pygame.Rect((pos[0] * BUTTON_DIMS[0], pos[1] * BUTTON_DIMS[1]),(BUTTON_DIMS[0] + 5, BUTTON_DIMS[1] * len(elements.keys()) + 5),),starting_layer_height=PANEL_LAYER,manager=manager,);self.pos = pos;self.path = path;self.elements = elements;self.loadfunc = loadfunc;ekeys = list(elements.keys());[UIButton(pygame.Rect((0, i * BUTTON_DIMS[1]), BUTTON_DIMS),text=ekeys[i],manager=manager,container=self,object_id="menu-" + self.path.replace(".", "-"),) for i in range(len(ekeys))]
    def process_event(self, event):
        if event.type != pygame.USEREVENT:return
        if event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == ("panel.menu-" + self.path.replace(".", "-")):uitext = event.ui_element.text
        if event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == ("panel.menu-" + self.path.replace(".", "-")) and self.elements[uitext] == None:self.loadfunc(self.path + "." + uitext)
        if event.user_type == pygame_gui.UI_BUTTON_ON_HOVERED and event.ui_object_id == ("panel.menu-" + self.path.replace(".", "-")):uitext = event.ui_element.text
        if event.user_type == pygame_gui.UI_BUTTON_ON_HOVERED and event.ui_object_id == ("panel.menu-" + self.path.replace(".", "-")) and self.elements[uitext] != None and self.child is not None:self.child.destroy()
        if event.user_type == pygame_gui.UI_BUTTON_ON_HOVERED and event.ui_object_id == ("panel.menu-" + self.path.replace(".", "-")) and self.elements[uitext] != None:self.child = Menu(self.ui_manager,(self.pos[0] + 1, list(self.elements.keys()).index(uitext)),self.path + "." + uitext,self.elements[uitext],self.loadfunc,)
    def destroy(self):
        if self.child is not None:self.child.destroy();self.child = None
        self.kill()
class OS:
    SCREEN = None;DIMS = None;BG = None;MANAGER = None;BG_COLOR = (255, 128, 0);PAINT = False;PAINT_RADIUS = 10;PAINT_COLOR = 0;PAINT_COLOR_LIST = [(255, 255, 255),(192, 192, 192),(128, 128, 128),(0, 0, 0),(0, 255, 0),(0, 128, 0),(128, 128, 0),(0, 128, 128),(255, 0, 0),(128, 0, 0),(128, 0, 128),(255, 0, 255),(0, 0, 255),(0, 0, 128),(0, 255, 255),(255, 255, 0),];PAINT_SHAPE = 0;NUM_SHAPES = 3;FOCUS = None;APPS = {};APPMENU = None
    def __init__(self):apps_path = os.path.dirname(os.path.abspath(__file__)) + "/apps";OS.iter_dir(self.APPS, apps_path);pygame.init();pygame.display.set_caption('rosehip');os.putenv("SDL_FBDEV", "/dev/fb0");pygame.display.init();self.DIMS = (pygame.display.Info().current_w, pygame.display.Info().current_h);self.SCREEN = pygame.display.set_mode(self.DIMS, pygame.FULLSCREEN);self.BG = pygame.Surface((self.DIMS));self.BG.fill(self.BG_COLOR);self.BRUSH_SURF = pygame.Surface((self.DIMS), flags=pygame.SRCALPHA);self.BRUSH_SURF.fill((0, 0, 0, 0));self.MANAGER = pygame_gui.UIManager(self.DIMS);pygame.mouse.set_visible(True);pygame.display.update()
    def iter_dir(tree, path):
        for f in os.listdir(path):
            if os.path.isfile(path + "/" + f + "/__init__.py"):tree[f] = None
            elif os.path.isdir(path + "/" + f):tree[f] = {};OS.iter_dir(tree[f], path + "/" + f)
    def appmenu_load(self, app,params=None):
        if self.APPMENU is not None:self.APPMENU.destroy();self.APPMENU = None
        importlib.import_module(app).load(self.MANAGER, params)
    def set_bg_color(self, color):self.BG = pygame.Surface((self.DIMS));self.BG_COLOR = color;self.BG.fill(self.BG_COLOR)
    def set_bg_image(self, file):
        filename, file_extension = os.path.splitext(file)
        if file_extension == ".jpg" or file_extension == ".png":self.BG = pygame.transform.scale(pygame.image.load(file), self.DIMS)
    def run(self):
        clock = pygame.time.Clock();running = True
        while running:
            delta = clock.tick(60) / 1000.0;pressed = pygame.key.get_pressed()
            for event in pygame.event.get():
                if event.type == pygame.KEYDOWN and event.key == pygame.K_HOME and self.APPMENU is None:self.APPMENU = Menu(self.MANAGER,(0, 0),os.path.dirname(os.path.abspath(__file__)) +"/apps",self.APPS,self.appmenu_load,)
                elif event.type == pygame.KEYDOWN and event.key == pygame.K_HOME:self.APPMENU.destroy();self.APPMENU = None
                elif event.type == pygame.KEYDOWN and event.key == pygame.K_INSERT:self.PAINT = not self.PAINT;self.BRUSH_SURF.fill((0, 0, 0, 0))
                if not event.type == pygame.KEYDOWN and event.type == pygame.MOUSEBUTTONDOWN and self.PAINT and event.button == 4 and  pressed[pygame.K_LALT]:self.PAINT_COLOR = (self.PAINT_COLOR + 1) % len(self.PAINT_COLOR_LIST)
                elif not event.type == pygame.KEYDOWN and event.type == pygame.MOUSEBUTTONDOWN and self.PAINT and event.button == 4 and pressed[pygame.K_LCTRL]:self.PAINT_SHAPE = (self.PAINT_SHAPE + 1) % self.NUM_SHAPES
                elif not event.type == pygame.KEYDOWN and event.type == pygame.MOUSEBUTTONDOWN and self.PAINT and event.button == 4:self.PAINT_RADIUS += 2
                if not event.type == pygame.KEYDOWN and event.type == pygame.MOUSEBUTTONDOWN and self.PAINT and event.button == 5 and event.button != 4 and pressed[pygame.K_LALT]:self.PAINT_COLOR = (self.PAINT_COLOR - 1) % len(self.PAINT_COLOR_LIST)
                elif not event.type == pygame.KEYDOWN and event.type == pygame.MOUSEBUTTONDOWN and self.PAINT and event.button == 5 and event.button != 4 and pressed[pygame.K_LCTRL]:self.PAINT_SHAPE = (self.PAINT_SHAPE - 1) % self.NUM_SHAPES
                elif not event.type == pygame.KEYDOWN and event.type == pygame.MOUSEBUTTONDOWN and self.PAINT and event.button == 5 and event.button != 4:self.PAINT_RADIUS -= 2
                if not event.type == pygame.KEYDOWN and event.type == pygame.MOUSEBUTTONDOWN and self.PAINT and self.PAINT_RADIUS < 2 and event.button == 5 and event.button != 4:self.PAINT_RADIUS = 2
                if event.type == pygame.USEREVENT and not event.type == pygame.KEYDOWN and not event.type == pygame.MOUSEBUTTONDOWN and event.user_type == "window_selected" and self.FOCUS is not None:self.FOCUS.unfocus()
                if event.type == pygame.USEREVENT and not event.type == pygame.KEYDOWN and not event.type == pygame.MOUSEBUTTONDOWN and event.user_type == "window_selected":self.FOCUS = event.ui_element;self.FOCUS.focus()
                elif event.type == pygame.USEREVENT and not event.type == pygame.KEYDOWN and not event.type == pygame.MOUSEBUTTONDOWN and event.user_type == pygame_gui.UI_COLOUR_PICKER_COLOUR_PICKED and event.ui_object_id == "#desktop_colour_picker":self.set_bg_color(event.colour[:-1])
                elif event.type == pygame.USEREVENT and not event.type == pygame.KEYDOWN and not event.type == pygame.MOUSEBUTTONDOWN and event.user_type == pygame_gui.UI_FILE_DIALOG_PATH_PICKED and event.ui_object_id == "#background_picker":self.set_bg_image(event.text)
                self.MANAGER.process_events(event)
            self.MANAGER.update(delta)
            if self.PAINT:mpos = pygame.mouse.get_pos();draw_surf = self.BRUSH_SURF
            if self.PAINT and pygame.mouse.get_pressed()[0]:draw_surf = self.BG
            if self.PAINT and self.PAINT_SHAPE == 0:pygame.draw.circle(draw_surf,self.PAINT_COLOR_LIST[self.PAINT_COLOR],mpos,self.PAINT_RADIUS,)
            elif self.PAINT and self.PAINT_SHAPE == 1:pygame.draw.rect(draw_surf,self.PAINT_COLOR_LIST[self.PAINT_COLOR],pygame.Rect((mpos[0] - self.PAINT_RADIUS, mpos[1] - self.PAINT_RADIUS),(self.PAINT_RADIUS * 2, self.PAINT_RADIUS * 2),),)
            elif self.PAINT and self.PAINT_SHAPE == 2:pygame.draw.polygon(draw_surf,self.PAINT_COLOR_LIST[self.PAINT_COLOR],((mpos[0] - self.PAINT_RADIUS, mpos[1] + self.PAINT_RADIUS),(mpos[0] + self.PAINT_RADIUS, mpos[1] + self.PAINT_RADIUS),(mpos[0], mpos[1] - self.PAINT_RADIUS),),)
            if self.PAINT:self.SCREEN.blit(self.BG, (0, 0));self.SCREEN.blit(self.BRUSH_SURF, (0, 0));self.BRUSH_SURF.fill((0, 0, 0, 0))
            if not self.PAINT:self.SCREEN.blit(self.BG, (0, 0))
            self.MANAGER.draw_ui(self.SCREEN);pygame.display.update()
def pc():OS().run()
def laptop():threading.Thread(target=lambda:[keyboard.send('alt+tab'),time.sleep(0.05),keyboard.send('alt+tab')]).start();OS().run()


import pygame;import pygame_gui;from pygame_gui.ui_manager import UIManager;from pygame_gui.elements.ui_window import UIWindow;from pygame_gui.elements.ui_image import UIImage;from pygame.locals import *;import math;import random
class Score:
    def __init__(self, font):self.player_1_score = 0;self.player_2_score = 0;self.font = font;self.score_string = None;self.score_text_render = None;self.update_score_text()
    def update_score_text(self):self.score_string = str(self.player_1_score) + " - " + str(self.player_2_score);self.score_text_render = self.font.render(self.score_string, True, pygame.Color(200, 200, 200))
    def render(self, screen, size):screen.blit(self.score_text_render,self.score_text_render.get_rect(centerx=size[0] / 2, centery=size[1] / 10),)
    def increase_player_1_score(self):self.player_1_score += 1;self.update_score_text()
    def increase_player_2_score(self):self.player_2_score += 1;self.update_score_text()
class ControlScheme:
    def __init__(self):self.up = K_UP;self.down = K_DOWN
class Bat:
    def __init__(self, start_pos, control_scheme, court_size):self.control_scheme = control_scheme;self.move_up = False;self.move_down = False;self.move_speed = 450.0;self.court_size = court_size;self.length = 30.0;self.width = 5.0;self.position = [float(start_pos[0]), float(start_pos[1])];self.rect = pygame.Rect((start_pos[0], start_pos[1]), (self.width, self.length));self.colour = pygame.Color("#FFFFFF")
    def process_event(self, event):
        if event.type == KEYDOWN and event.key == self.control_scheme.up:self.move_up = True
        if event.type == KEYDOWN and event.key == self.control_scheme.down:self.move_down = True
        if event.type == KEYUP and event.key == self.control_scheme.up:self.move_up = False
        if event.type == KEYUP and event.key == self.control_scheme.down:self.move_down = False
    def update(self, dt):
        if self.move_up:self.position[1] -= dt * self.move_speed
        if self.move_up and self.position[1] < 10.0:self.position[1] = 10.0
        if self.move_up:self.rect.y = self.position[1]
        if self.move_down:self.position[1] += dt * self.move_speed
        if self.move_down and self.position[1] > self.court_size[1] - self.length - 10:self.position[1] = self.court_size[1] - self.length - 10
        if self.move_down:self.rect.y = self.position[1]
    def render(self, screen):pygame.draw.rect(screen, self.colour, self.rect)
class Ball:
    def __init__(self, start_position):self.rect = pygame.Rect(start_position, (5, 5));self.colour = pygame.Color(255, 255, 255);self.position = [float(start_position[0]), float(start_position[1])];self.start_position = [self.position[0], self.position[1]];self.ball_speed = 120.0;self.max_bat_bounce_angle = 5.0 * math.pi / 12.0;self.collided = False;self.velocity = [0.0, 0.0];self.create_random_start_vector()
    def render(self, screen):pygame.draw.rect(screen, self.colour, self.rect)
    def create_random_start_vector(self):
        y_random = random.uniform(-0.5, 0.5);x_random = 1.0 - abs(y_random)
        if random.randint(0, 1) == 1:x_random = x_random * -1.0
        self.velocity = [x_random * self.ball_speed, y_random * self.ball_speed]
    def reset(self):self.position = [self.start_position[0], self.start_position[1]];self.create_random_start_vector()
    def update(self, dt, bats, walls):
        self.position[0] += self.velocity[0] * dt;self.position[1] += self.velocity[1] * dt;self.rect.x = self.position[0];self.rect.y = self.position[1];collided_this_frame = False
        for wall in walls:
            if self.rect.colliderect(wall.rect):collided_this_frame = True
            if self.rect.colliderect(wall.rect) and not self.collided:self.collided = True;self.velocity[1] = self.velocity[1] * -1
        for bat in bats:
            if self.rect.colliderect(bat.rect):collided_this_frame = True
            if self.rect.colliderect(bat.rect) and not self.collided:self.collided = True;bat_y_centre = bat.position[1] + (bat.length / 2);ball_y_centre = self.position[1] + 5;relative_intersect_y = (bat_y_centre - ball_y_centre);normalized_relative_intersect_y = relative_intersect_y / (bat.length / 2);bounce_angle = (normalized_relative_intersect_y * self.max_bat_bounce_angle);self.velocity[0] = self.velocity[0] * -1;self.velocity[1] = self.ball_speed * -math.sin(bounce_angle)
        if not collided_this_frame:self.collided = False
class Wall:
    def __init__(self, top_left, bottom_right):self.rect = pygame.Rect(top_left, (bottom_right[0] - top_left[0], bottom_right[1] - top_left[1]));self.colour = pygame.Color("#C8C8C8")
    def render(self, screen):pygame.draw.rect(screen, self.colour, self.rect)
class PongGame:
    def __init__(self, size):self.size = size;self.background = pygame.Surface(size);self.background = self.background.convert();self.background.fill((0, 0, 0));font = pygame.font.Font(None, 24);self.score = Score(font);self.walls = [Wall((5, 5), (size[0] - 10, 10)),Wall((5, size[1] - 10), (size[0] - 10, size[1] - 5)),];self.bats = [];control_scheme_1 = ControlScheme();control_scheme_1.up = K_w;control_scheme_1.down = K_s;control_scheme_2 = ControlScheme();control_scheme_2.up = K_UP;control_scheme_2.down = K_DOWN;self.bats.append(Bat((5, int(size[1] / 2)), control_scheme_1, self.size));self.bats.append(Bat((size[0] - 10, int(size[1] / 2)), control_scheme_2, self.size));self.ball = Ball((int(size[0] / 2), int(size[1] / 2)))
    def process_event(self, event):
        for bat in self.bats:bat.process_event(event)
    def update(self, time_delta):
        for bat in self.bats:bat.update(time_delta)
        self.ball.update(time_delta, self.bats, self.walls)
        if self.ball.position[0] < 0:self.ball.reset();self.score.increase_player_2_score()
        elif self.ball.position[0] > self.size[0]:self.ball.reset();self.score.increase_player_1_score()
    def draw(self, surface):
        surface.blit(self.background, (0, 0))
        for wall in self.walls:wall.render(surface)
        for bat in self.bats:bat.render(surface)
        self.ball.render(surface);self.score.render(surface, self.size)
class PongWindow(UIWindow):
    def __init__(self, position, ui_manager):super().__init__(pygame.Rect(position, (320, 240)),ui_manager,window_display_title="pong",object_id="#pong_window",);game_surface_size = self.get_container().get_size();self.game_surface_element = UIImage(pygame.Rect((0, 0), game_surface_size),pygame.Surface(game_surface_size).convert(),manager=ui_manager,container=self,parent_element=self,);self.pong_game = PongGame(game_surface_size);self.is_active = False
    def focus(self):self.is_active = True
    def unfocus(self):self.is_active = False
    def process_event(self, event):
        handled = super().process_event(event)
        if event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#pong_window.#title_bar" and event.ui_element == self.title_bar:handled = True;event_data = {"user_type": "window_selected","ui_element": self,"ui_object_id": self.most_specific_combined_id,};window_selected_event = pygame.event.Event(pygame.USEREVENT, event_data);pygame.event.post(window_selected_event)
        if self.is_active:handled = self.pong_game.process_event(event)
        return handled
    def update(self, time_delta):
        if self.alive() and self.is_active:self.pong_game.update(time_delta)
        super().update(time_delta)
        self.pong_game.draw(self.game_surface_element.image)
def load(manager, params):
    pos = (50, 50)
    if params is not None and len(params) > 0:pos = params[0]
    PongWindow(pos, manager)



import os;import pygame;import pygame_gui;from pygame_gui.elements.ui_image import UIImage;from math import floor
class Dance(pygame_gui.elements.UIWindow):
    def __init__(self, pos, manager):super().__init__(pygame.Rect(pos, (352,300)),manager=manager,window_display_title="animation",object_id="#animation",);self.FRAMES = [];self.FRAMES_LEN = 0;self.FRAME_INDEX = 0;self.dsurf = UIImage(pygame.Rect((0, 0),(320, 240)),pygame.Surface((320, 240)).convert(),manager=manager,container=self,parent_element=self,);app_path = os.path.dirname(os.path.abspath(__file__));frames_path = app_path + "/frames/";[self.FRAMES.append(pygame.transform.scale(pygame.image.load(frames_path + "frame-" + str(floor(x/2)) + ".png"),(320, 240))) for x in range(len([name for name in os.listdir('.') if os.path.isfile(name)])*2-1)];self.FRAMES_LEN = len(self.FRAMES)
    def update(self, delta):super().update(delta);self.dsurf.image.blit(self.FRAMES[self.FRAME_INDEX], (0, 0));self.FRAME_INDEX = (self.FRAME_INDEX + 1) % self.FRAMES_LEN
def load(manager, params):
    pos = (100, 100)
    if params is not None and len(params) > 0:pos = params[0]
    Dance(pos, manager)



def load(manager, params):
    pos = (100, 100);import pygame;import pygame_gui
    if params is not None and len(params) > 0:pos = params[0]
    pygame_gui.windows.UIColourPickerDialog(rect=pygame.Rect(pos, (600, 400)),manager=manager,window_title="Set Background Color",object_id="#desktop_colour_picker",)



import requests,html2text;import sys;import io;import os;os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide";import pygame;from io import StringIO;pygame.init();pygame.display.set_caption('internet');font = pygame.font.SysFont('arial', 32);clock = pygame.time.Clock();input_box = pygame.Rect(100, 100, 140, 32);out_box=pygame.Rect(100,200, 140, 32);from pygame_gui.elements import UIWindow;from pygame_gui.elements import UITextBox;from pygame_gui.elements import UITextEntryLine;from pygame_gui.elements import UITextBox;import pygame_gui;import concurrent.futures
class internet(UIWindow):
    def __init__(self, pos, manager):super().__init__(pygame.Rect(pos, (400, 300)), manager, window_display_title="web", object_id="#web",resizable=True);self.textbox = pygame_gui.elements.UITextBox("",relative_rect=pygame.Rect(0, 0, 368, 200),manager=manager,container=self,anchors={"left": "left","right": "right","top": "top","bottom": "bottom",},);self.input = pygame_gui.elements.UITextEntryLine(relative_rect=pygame.Rect(0, -35, 368, 30),manager=manager,container=self,anchors={"left": "left","right": "right","top": "bottom","bottom": "bottom",},);self.text='';self.manager=manager;self.input.focus()
    def process_event(self, event):
        super().process_event(event)
        if event.type == pygame.KEYUP and event.key == pygame.K_RETURN:self.text=html2text.html2text(requests.get(('www' in self.input.get_text() and 'http' not in self.input.get_text())*'https://'+('www' not in self.input.get_text())*'www.google.com/search?lang=en&q='+self.input.get_text()).text).replace('\n','<br>');self.input.kill();self.textbox.kill();self.textbox = pygame_gui.elements.UITextBox(self.text,relative_rect=pygame.Rect(0, 0, 368, 200),manager=self.manager,container=self,anchors={"left": "left","right": "right","top": "top","bottom": "bottom",},);self.input = pygame_gui.elements.UITextEntryLine(relative_rect=pygame.Rect(0, -35, 368, 30),manager=self.manager,container=self,anchors={"left": "left","right": "right","top": "bottom","bottom": "bottom",},);self.input.focus()
def load(manager, params):
    pos = (100, 100)
    if params is not None and len(params) > 0:pos = params[0]
    internet(pos, manager)



import pygame;import os;import pygame_gui;from pygame_gui.elements.ui_image import UIImage;import cv2;import numpy;camera=cv2.VideoCapture(0,cv2.CAP_DSHOW);camera.set(3,640);camera.set(4,480)
class Camera(pygame_gui.elements.UIWindow):
    def __init__(self, pos, manager):super().__init__(pygame.Rect(pos, (352, 300)),manager=manager,window_display_title="camera",object_id="#camera",);self.dsurf = UIImage(pygame.Rect((0, 0),(320,240)),pygame.Surface((320,240)).convert(),manager=manager,container=self,parent_element=self,)
    def update(self, delta):super().update(delta);self.dsurf.image.blit(pygame.surfarray.make_surface(numpy.rot90(cv2.cvtColor(camera.read()[1],cv2.COLOR_BGR2RGB))), (0, 0))
def load(x,y):
    pos = (100, 100)
    if y is not None and len(y) > 0:pos = y[0]
    Camera(pos, x)



import glob;import pygame;import pygame_gui;import os;from pygame import mixer
class Mixer(object):
    def __init__(self):mixer.init()
    @classmethod
    def load(cls, file):mixer.music.load(file)
    @classmethod
    def play(cls):mixer.music.play()
    @classmethod
    def pause(cls):cls.isPaused = True;mixer.music.pause()
    @classmethod
    def resume(cls):mixer.music.unpause()
    @classmethod
    def rewind(cls):mixer.music.rewind()
    @classmethod
    def stop(cls):mixer.music.stop()
    @classmethod
    def set_volume(cls, value):mixer.music.set_volume(value)
    @classmethod
    def get_volume(cls):return float(mixer.music.get_volume())
class MusicPlayer(pygame_gui.elements.UIWindow):
    DIMS = (400, 250)
    def __init__(self, pos, manager):
        super().__init__(pygame.Rect(pos, (self.DIMS[0] + 32, self.DIMS[1] + 60)),manager=manager,window_display_title="MusicPlayer",object_id="#musicplayer",);self.mixer = Mixer();app_path = '\\'.join(os.path.dirname(os.path.realpath(__file__)).split('\\')[:-3]);self.musics_path = app_path + "/musics";self.musics = glob.glob(self.musics_path + "/*.ogg");self.musics_dict = dict((music_path.split("/")[-1], music_path) for music_path in self.musics);self.isPaused = False;btns_names = ["play", "pause", "stop", "+", "-"];self.BSIZE = (self.DIMS[0] / len(btns_names), 30)
        for i, btn in enumerate(btns_names):self.play_button = pygame_gui.elements.UIButton(relative_rect=pygame.Rect((i % len(btns_names) * self.BSIZE[0], self.DIMS[1] - self.BSIZE[1]),self.BSIZE,),text=btn.upper(),manager=manager,container=self,object_id="#op-" + btn + "btn",);self.music_list = pygame_gui.elements.UISelectionList(relative_rect=pygame.Rect((0, 0), (self.DIMS[0], self.DIMS[1] - self.BSIZE[1])),manager=manager,container=self,object_id="#op-musiclist",item_list=list(self.musics_dict.keys()),)
    def process_event(self, event):
        super().process_event(event)
        if event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op-playbtn":selected_music = self.music_list.get_single_selection()
        if event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op-playbtn" and selected_music is None:music_path = next(iter(self.musics_dict.values()))
        elif event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op-playbtn":music_path = self.musics_dict[selected_music]
        if event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op-playbtn" and self.isPaused:self.mixer.resume();self.isPaused = False
        elif event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op-playbtn":self.mixer.load(music_path);self.mixer.play()
        elif event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op-pausebtn":self.mixer.pause();self.isPaused = True
        elif event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op-stopbtn":self.mixer.stop()
        elif event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op-+btn":volume = min((self.mixer.get_volume() + 0.1), 1.0);self.mixer.set_volume(value=volume)
        elif event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED and event.ui_object_id == "#musicplayer.#op--btn":volume = max((self.mixer.get_volume() - 0.1), 0.0);self.mixer.set_volume(value=volume)
        if event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED:return True
    def update(self, delta):super().update(delta)
    def kill(self):self.mixer.stop();super().kill()
def load(manager, params):
    pos = (100, 100)
    if params is not None and len(params) > 0:pos = params[0]
    MusicPlayer(pos, manager)



import threading;import pygame;import pygame_gui;from pygame_gui.elements import UILabel;from pygame_gui.elements import UITextEntryLine;import pyttsx3
class Speaker(pygame_gui.elements.UIWindow):
    def __init__(self, pos, manager):super().__init__(pygame.Rect(pos, (400, 128)),manager=manager,window_display_title="speaker",object_id="#speaker",);self.label = UILabel(relative_rect=pygame.Rect(-20, 10, 400, 20),text="",manager=manager,container=self,);self.input = UITextEntryLine(relative_rect=pygame.Rect(0, 40, 368, 30), manager=manager, container=self);self.engine = pyttsx3.init();self.engine.setProperty("rate", 150);self.speakthrd = None
    def process_event(self, event):
        super().process_event(event)
        if event.type == pygame.USEREVENT and event.ui_element == self.input and event.user_type == pygame_gui.UI_TEXT_ENTRY_FINISHED and (self.speakthrd is None or not self.speakthrd.is_alive()) and self.input.get_text!="":self.engine.say(self.input.get_text);self.speakthrd = threading.Thread(target=self.engine.runAndWait, args=());self.speakthrd.start();self.label.set_text(self.input.get_text);self.input.set_text("")
def load(manager, params):
    pos = (100, 100)
    if params is not None and len(params) > 0:pos = params[0]
    Speaker(pos, manager)



import pygame;import pygame_gui;from pygame_gui.elements.ui_image import UIImage;from math import pi, cos, sin;import datetime;DARK = 33, 41, 46;WHITE = 255, 255, 255;GRAY = 76, 80, 82;RED = 255, 0, 0
class SnakeAClock(pygame_gui.elements.UIWindow):
    DIMS = (300, 300)
    def __init__(self, pos, manager):super().__init__(pygame.Rect(pos, (self.DIMS[0] + 32, self.DIMS[1] + 60)),manager=manager,window_display_title="clock",object_id="#aclock",resizable=True,);self.dsurf = UIImage(pygame.Rect((0, 0), self.DIMS),pygame.Surface(self.DIMS).convert(),manager=manager,container=self,parent_element=self,);self.draw_dial();self.clean_dial = self.dial.copy();self.manager = manager
    def process_event(self, event):
        super().process_event(event);r = super().get_abs_rect()
        if event.type == pygame.MOUSEBUTTONUP and (r.w != self.DIMS[0] + 32 or r.h != self.DIMS[1] + 60):self.DIMS = r.w - 32, r.h - 60;super().kill();self.__init__((r.left, r.top), self.manager)
    def update(self, delta):super().update(delta);self.dial = self.clean_dial.copy();self.draw_hands();self.dsurf.image.blit(self.dial, (0, 0))
    def clocksize(self):return int(0.95 * min(self.DIMS) / 2)
    def diam(self, x, r1, r2, r3, dx):x1, y1 = (r1 * self.clocksize() * sin(x * pi / 180),r1 * self.clocksize() * cos(x * pi / 180),);x2, y2 = (r2 * self.clocksize() * sin((x + dx) * pi / 180),r2 * self.clocksize() * cos((x + dx) * pi / 180),);x3, y3 = (r3 * self.clocksize() * sin(x * pi / 180),r3 * self.clocksize() * cos(x * pi / 180),);x4, y4 = (r2 * self.clocksize() * sin((x - dx) * pi / 180),r2 * self.clocksize() * cos((x - dx) * pi / 180),);p = [(int(x1 + self.DIMS[0] // 2), int(-y1 + self.DIMS[1] // 2)),(int(x2 + self.DIMS[0] // 2), int(-y2 + self.DIMS[1] // 2)),(int(x3 + self.DIMS[0] // 2), int(-y3 + self.DIMS[1] // 2)),(int(x4 + self.DIMS[0] // 2), int(-y4 + self.DIMS[1] // 2)),];pygame.draw.polygon(self.dial, WHITE, p)
    def draw_dial(self):
        self.dial = pygame.Surface(self.DIMS);self.dial.fill(DARK);pygame.draw.circle(self.dial, WHITE, (self.DIMS[0] // 2, self.DIMS[1] // 2), self.clocksize());pygame.draw.circle(self.dial,GRAY,(self.DIMS[0] // 2, self.DIMS[1] // 2),int(0.99 * self.clocksize()),)
        for x in range(60):
            if x % 5 == 0:continue
            x1, y1 = (0.95 * self.clocksize() * sin(6 * x * pi / 180),0.95 * self.clocksize() * cos(6 * x * pi / 180),);x2, y2 = (0.92 * self.clocksize() * sin(6 * x * pi / 180),0.92 * self.clocksize() * cos(6 * x * pi / 180),);pygame.draw.line(self.dial,WHITE,(int(x1 + self.DIMS[0] // 2), int(y1 + self.DIMS[1] // 2)),(int(x2 + self.DIMS[0] // 2), int(y2 + self.DIMS[1] // 2)),2,)
        for x in range(0, 360, 30):
            if x % 90 == 0:self.diam(x, 0.95, 0.88, 0.88, 2)
            else:self.diam(x, 0.95, 0.95, 0.87, 1)
    def draw_hands(self):now = datetime.datetime.now();x = 30 * ((now.hour - 12) % 12) + now.minute / 2;self.diam(x, 0.45, 0.08, -0.08, 30);x = 6 * now.minute + now.second / 10;self.diam(x, 0.75, 0.08, -0.08, 25);x = 6 * now.second;x1, y1 = (0.8 * self.clocksize() * sin(x * pi / 180),0.8 * self.clocksize() * cos(x * pi / 180),);x2, y2 = 0, 0;pygame.draw.line(self.dial,RED,(int(x1 + self.DIMS[0] // 2), int(-y1 + self.DIMS[1] // 2)),(int(x2 + self.DIMS[0] // 2), int(-y2 + self.DIMS[1] // 2)),3,)
def load(manager, params):
    pos = (100, 100)
    if params is not None and len(params) > 0:pos = params[0]
    SnakeAClock(pos, manager)



def load(manager, params):
    pos = (30, 30);import pygame;import pygame_gui;import os
    if params is not None and len(params) > 0:pos = params[0]
    pygame_gui.windows.UIFileDialog(rect=pygame.Rect(pos, (600, 400)),manager=manager,window_title="Set Background Image",initial_file_path='\\'.join(os.path.dirname(os.path.realpath(__file__)).split('\\')[:-3])+'/images',object_id="#background_picker",)



import os,cv2;import pygame;import pygame_gui;from pygame_gui.elements.ui_image import UIImage;from math import floor;import numpy
class Video(pygame_gui.elements.UIWindow):
    def __init__(self, pos, manager):super().__init__(pygame.Rect(pos, (352,300)),manager=manager,window_display_title="Video",object_id="#video",);self.dsurf = UIImage(pygame.Rect((0, 0),(320, 240)),pygame.Surface((320, 240)).convert(),manager=manager,container=self,parent_element=self,);self.video=cv2.VideoCapture(os.path.dirname(os.path.abspath(__file__))+'\\video.mp4')
    def update(self, delta):
        super().update(delta)
        if not self.video.read()[0]:self.video=cv2.VideoCapture(os.path.dirname(os.path.abspath(__file__))+'\\video.mp4')
        else:self.dsurf.image.blit(pygame.surfarray.make_surface(numpy.rot90(cv2.cvtColor(cv2.cvtColor(self.video.read()[1],cv2.COLOR_RGB2BGR),cv2.COLOR_BGR2RGB))), (0, 0))
def load(manager, params):
    pos = (100, 100)
    if params is not None and len(params) > 0:pos = params[0]
    Video(pos, manager)



from setuptools import setup,find_packages
setup(
    name='rosehip',
    include_package_data=True,
    version='1.0.2',
    description=open('README.md').readlines()[0].replace('\n', ''),
    long_description='\n'.join([i.replace('\n', '') for i in open('README.md').readlines()[1:]]),
    long_description_content_type="text/markdown",
    packages=find_packages(),
    url='https://github.com/donno2048/Rosehip-pypi',
    package_data={'rosehip': ['Rosehip'],},
    license='MIT',
    author='Elisha Hollander',
    classifiers=['Operating System :: Microsoft :: Windows','License :: OSI Approved :: MIT License','Programming Language :: Python :: 3'],
    install_requires='pygame,pyttsx3,pywin32,comtypes,pygame-gui,keyboard,wheel,Js2Py,selenium,chromedriver-autoinstaller,html2text,opencv-python'.split(',')
)



import pygame;import pygame_gui
class SnakeCalc(pygame_gui.elements.UIWindow):
    OPS = "789+" "456-" "123*" "p0=/" "C";BSIZE = (67, 75);USERCALC = ""
    def __init__(self, pos, manager):
        super().__init__(pygame.Rect(pos, (300, 475)),manager=manager,window_display_title="snakecalc",object_id="#snaketerm",resizable=True,);self.textbox = pygame_gui.elements.UITextBox("",relative_rect=pygame.Rect(0, 1, 268, 40),manager=manager,container=self,anchors={"left": "left","right": "right","top": "top","bottom": "bottom",},)
        for i in range(len(self.OPS)):
            op = self.OPS[i]
            if op == "x":continue
            pygame_gui.elements.UIButton(relative_rect=pygame.Rect((i % 4 * self.BSIZE[0], 40 + int(i / 4) * self.BSIZE[1]), self.BSIZE),text="." if op == "p" else op,manager=manager,container=self,object_id="#op-" + op,)
    def process_event(self, event):
        super().process_event(event)
        if event.type == pygame.USEREVENT and event.user_type == pygame_gui.UI_BUTTON_PRESSED:self.input_op(event.ui_object_id[-1]);return True
    def set_text(self, text):self.textbox.html_text = text;self.textbox.rebuild()
    def append_text(self, text):self.textbox.html_text = self.textbox.html_text + text;self.textbox.rebuild()
    def calculate(self, expression):
        result = ""
        try:result = str(eval(expression))
        except Exception:result = "Error"
        self.set_text(result)
    def input_op(self, op):
        if op in self.OPS and op == "=":self.calculate(self.textbox.html_text)
        elif op in self.OPS and op == "C":self.set_text("")
        elif op in self.OPS and op == "p":self.append_text(".")
        elif op in self.OPS:self.append_text(op)
def load(manager, params):
    pos = (100, 100)
    if params is not None and len(params) > 0:pos = params[0]
    SnakeCalc(pos, manager)
